/*
  ==============================================================================

   This file is part of the Plugin Consultant - VST API Logger
   Copyright 2005 by Ofer Shem Tov.

  ------------------------------------------------------------------------------

   Plugin Consultant can be redistributed and/or modified under the terms of the
   GNU General Public License, as published by the Free Software Foundation;
   either version 2 of the License, or (at your option) any later version.

   Plugin Consultant is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Plugin Consultant; if not, visit www.gnu.org/licenses or write to the
   Free Software Foundation, Inc., 59 Temple Place, Suite 330,
   Boston, MA 02111-1307 USA

  ==============================================================================
*/

#ifdef _MSC_VER
#pragma warning (disable : 4996)
#endif

#include <AEffectx.h>
#include <juce.h>
#include "PCLogger.h"

extern AEffect proxyEffect;
extern PCLogger pcLogger;
extern File lastDir;
extern void copyAEffect();

static audioMasterCallback vstAudioMaster = NULL;
static String path;
static void* pluginHandle = NULL;

long VSTCALLBACK proxyAudioMaster(AEffect *effect, long opcode, long index, long value, void *ptr, float opt);

//-----------------------------------------------------------------------------------------
AEffect* loadVstPlugin(audioMasterCallback audioMaster)
{
	// remember original audio master
	vstAudioMaster = audioMaster;

	if ( !lastDir.isDirectory() )
		lastDir = SystemStats::getUserHomeDirectory();

	FileChooser myChooser ("Please select the plugin to log...", lastDir, "*.dll");

	// choose plugin to load
	if (myChooser.browseForFileToOpen())
	{
		File pluginFile	(myChooser.getResult());
		lastDir = pluginFile.getParentDirectory();
		path = pluginFile.getFullPathName();

		// set logger name to this plug name
		pcLogger.setPlugName( pluginFile.getFileName() );
	}
	else
	{
		pcLogger.log("No plugin was selected");
		return NULL;
	}

	// load the plugin library
	pluginHandle = Process::loadDynamicLibrary( path );

	if ( pluginHandle == NULL )
	{
		pcLogger.log("Failed to load " + path );
		return NULL;
	}

	pcLogger.log("Loaded " + path );

	AEffect* (/*__stdcall*/* getNewPlugInstance)(audioMasterCallback);
	getNewPlugInstance=(AEffect*(/*__stdcall*/*)(audioMasterCallback))Process::getProcedureEntryPoint(pluginHandle, "main");

	AEffect* newPlugin = NULL;

	if( getNewPlugInstance != NULL )
	{
		// get the plugin effect object and give it our proxy audio master
		newPlugin = getNewPlugInstance(proxyAudioMaster);
			
		// make sure it is a VST plugin
		if( newPlugin != NULL )
			if( newPlugin->magic!=kEffectMagic )
			{
				newPlugin = NULL;	// its not a VST
				pcLogger.log("Wrong Magic, probably not a VST plugin");
			}
	}
	else
		pcLogger.log("Failed to get main()");


	// if file loaded but plugin failed then unload the file
	if ( newPlugin == NULL )
	{
		Process::freeDynamicLibrary( pluginHandle );
		pcLogger.log("Unloaded " + path );
	}

	return newPlugin;
}

//-----------------------------------------------------------------------------------------
void	unloadPlugin()
{
	if ( pluginHandle != NULL )
	{
		pcLogger.log("Unloaded " + path );
		Process::freeDynamicLibrary( pluginHandle );
	}
}

//-----------------------------------------------------------------------------------------
long VSTCALLBACK proxyAudioMaster(AEffect *effect, long opcode, long index, long value, void *ptr, float opt)
{
	// the effect might changed its AEffect structure before calling the host (e.g. audioMasterIOChanged)
	void copyAEffect();

	pcLogger.logVstAudioMaster(opcode,index,value,ptr,opt);
	long retval = vstAudioMaster(&proxyEffect,opcode,index,value,ptr,opt);
	pcLogger.logVstAudioMasterReturn( opcode, ptr, retval );
	return retval;
}
